/**********************************/
/*                                */
/*  Copyright 2000, David Grant   */
/*                                */
/*  see LICENSE for more details  */
/*                                */
/**********************************/


struct _Address {
	short Size;
	char Mode;
	char Register;
	unsigned long Address;
	unsigned long Data;
};


/*ng Addressing_Retr(short Size, char Mode, char Register, char EAValue);
void Addressing_Stor(short Size, char Mode, char Register, long Value);*/
long Addressing_Print(short Size, char Mode, char Register, char *Str);

char EA_GetFromPC(struct _Address *Addr, short Size, char Mode, char Register);
char EA_GetValue(unsigned long *Result,  struct _Address *Addr);
char EA_GetEA(unsigned long *Result, struct _Address *Addr);
/*long EA_GetAddress(struct _Address *Addr);*/
void EA_PutValue(struct _Address *Addr, unsigned long Value);

void Stack_Push(short Size, unsigned long Value);
unsigned long Stack_Pop(short Size);

